Module: LaunchDarkly::Interfaces::DataSystem::EventName

Defined in:
lib/ldclient-rb/interfaces/data_system.rb

Overview

EventName represents the name of an event that can be sent by the server for FDv2.

Constant Summary collapse

PUT_OBJECT =

Specifies that an object should be added to the data set with upsert semantics.

:"put-object"
DELETE_OBJECT =

Specifies that an object should be removed from the data set.

:"delete-object"
SERVER_INTENT =

Specifies the server’s intent.

:"server-intent"
PAYLOAD_TRANSFERRED =

Specifies that all data required to bring the existing data set to a new version has been transferred.

:"payload-transferred"
HEARTBEAT =

Keeps the connection alive.

:"heart-beat"
GOODBYE =

Specifies that the server is about to close the connection.

:goodbye
ERROR =

Specifies that an error occurred while serving the connection.

:error