Module: LaunchDarkly::Impl::DataSystem::DiagnosticAccumulator Private
- Defined in:
- lib/ldclient-rb/impl/data_system.rb
Overview
This module is part of a private API. You should avoid using this module if possible, as it may be removed or be changed in the future.
Mixin that defines the required methods of a diagnostic accumulator implementation. The diagnostic accumulator is used for collecting and reporting diagnostic events to LaunchDarkly for monitoring SDK performance and behavior.
Application code should not need to implement this directly; it is used internally by the SDK.
Instance Method Summary collapse
-
#create_event_and_reset(dropped_events, deduplicated_users) ⇒ Object
private
Create an event and reset the accumulator.
-
#record_events_in_batch(events_in_batch) ⇒ void
private
Record events in a batch.
-
#record_stream_init(timestamp, duration, failed) ⇒ void
private
Record a stream initialization.
Instance Method Details
#create_event_and_reset(dropped_events, deduplicated_users) ⇒ Object
This method is part of a private API. You should avoid using this method if possible, as it may be removed or be changed in the future.
Create an event and reset the accumulator.
190 191 192 |
# File 'lib/ldclient-rb/impl/data_system.rb', line 190 def create_event_and_reset(dropped_events, deduplicated_users) raise NotImplementedError, "#{self.class} must implement #create_event_and_reset" end |
#record_events_in_batch(events_in_batch) ⇒ void
This method is part of a private API. You should avoid using this method if possible, as it may be removed or be changed in the future.
This method returns an undefined value.
Record events in a batch.
179 180 181 |
# File 'lib/ldclient-rb/impl/data_system.rb', line 179 def record_events_in_batch(events_in_batch) raise NotImplementedError, "#{self.class} must implement #record_events_in_batch" end |
#record_stream_init(timestamp, duration, failed) ⇒ void
This method is part of a private API. You should avoid using this method if possible, as it may be removed or be changed in the future.
This method returns an undefined value.
Record a stream initialization.
169 170 171 |
# File 'lib/ldclient-rb/impl/data_system.rb', line 169 def record_stream_init(, duration, failed) raise NotImplementedError, "#{self.class} must implement #record_stream_init" end |