Class: LaunchDarkly::Impl::Integrations::Redis::RedisFeatureStoreCore
- Inherits:
-
RedisStoreImplBase
- Object
- RedisStoreImplBase
- LaunchDarkly::Impl::Integrations::Redis::RedisFeatureStoreCore
- Defined in:
- lib/ldclient-rb/impl/integrations/redis_impl.rb
Overview
Internal implementation of the Redis feature store, intended to be used with CachingStoreWrapper.
Instance Method Summary collapse
- #available? ⇒ Boolean
- #description ⇒ Object
- #get_all_internal(kind) ⇒ Object
- #get_internal(kind, key) ⇒ Object
- #init_internal(all_data) ⇒ Object
-
#initialize(opts) ⇒ RedisFeatureStoreCore
constructor
A new instance of RedisFeatureStoreCore.
- #initialized_internal? ⇒ Boolean
- #upsert_internal(kind, new_item) ⇒ Object
Methods inherited from RedisStoreImplBase
Constructor Details
#initialize(opts) ⇒ RedisFeatureStoreCore
Returns a new instance of RedisFeatureStoreCore.
159 160 161 162 163 |
# File 'lib/ldclient-rb/impl/integrations/redis_impl.rb', line 159 def initialize(opts) super(opts) @test_hook = opts[:test_hook] # used for unit tests, deliberately undocumented end |
Instance Method Details
#available? ⇒ Boolean
165 166 167 168 169 170 171 |
# File 'lib/ldclient-rb/impl/integrations/redis_impl.rb', line 165 def available? # We don't care what the status is, only that we can connect initialized_internal? true rescue false end |
#description ⇒ Object
173 174 175 |
# File 'lib/ldclient-rb/impl/integrations/redis_impl.rb', line 173 def description "RedisFeatureStore" end |
#get_all_internal(kind) ⇒ Object
200 201 202 203 204 205 206 207 208 209 |
# File 'lib/ldclient-rb/impl/integrations/redis_impl.rb', line 200 def get_all_internal(kind) fs = {} with_connection do |redis| hashfs = redis.hgetall(items_key(kind)) hashfs.each do |k, json_item| fs[k.to_sym] = Model.deserialize(kind, json_item) end end fs end |
#get_internal(kind, key) ⇒ Object
194 195 196 197 198 |
# File 'lib/ldclient-rb/impl/integrations/redis_impl.rb', line 194 def get_internal(kind, key) with_connection do |redis| get_redis(redis, kind, key) end end |
#init_internal(all_data) ⇒ Object
177 178 179 180 181 182 183 184 185 186 187 188 189 190 191 192 |
# File 'lib/ldclient-rb/impl/integrations/redis_impl.rb', line 177 def init_internal(all_data) count = 0 with_connection do |redis| redis.multi do |multi| all_data.each do |kind, items| multi.del(items_key(kind)) count = count + items.count items.each do |key, item| multi.hset(items_key(kind), key, Model.serialize(kind,item)) end end multi.set(inited_key, inited_key) end end @logger.info { "RedisFeatureStore: initialized with #{count} items" } end |
#initialized_internal? ⇒ Boolean
243 244 245 |
# File 'lib/ldclient-rb/impl/integrations/redis_impl.rb', line 243 def initialized_internal? with_connection { |redis| redis.exists?(inited_key) } end |
#upsert_internal(kind, new_item) ⇒ Object
211 212 213 214 215 216 217 218 219 220 221 222 223 224 225 226 227 228 229 230 231 232 233 234 235 236 237 238 239 240 241 |
# File 'lib/ldclient-rb/impl/integrations/redis_impl.rb', line 211 def upsert_internal(kind, new_item) base_key = items_key(kind) key = new_item[:key] try_again = true final_item = new_item while try_again try_again = false with_connection do |redis| redis.watch(base_key) do old_item = get_redis(redis, kind, key) before_update_transaction(base_key, key) if old_item.nil? || old_item[:version] < new_item[:version] result = redis.multi do |multi| multi.hset(base_key, key, Model.serialize(kind, new_item)) end if result.nil? @logger.debug { "RedisFeatureStore: concurrent modification detected, retrying" } try_again = true end else final_item = old_item action = new_item[:deleted] ? "delete" : "update" # rubocop:disable Layout/LineLength @logger.warn { "RedisFeatureStore: attempted to #{action} #{key} version: #{old_item[:version]} in '#{kind[:namespace]}' with a version that is the same or older: #{new_item[:version]}" } end redis.unwatch end end end final_item end |