Module: LaunchDarkly::Interfaces::DataSystem::DataStoreMode
- Defined in:
- lib/ldclient-rb/interfaces/data_system.rb
Overview
DataStoreMode represents the mode of operation of a Data Store in FDV2 mode.
Constant Summary collapse
- READ_ONLY =
Indicates that the data store is read-only. Data will never be written back to the store by the SDK.
:read_only- READ_WRITE =
Indicates that the data store is read-write. Data from initializers/synchronizers may be written to the store as necessary.
:read_write